@@ -1,13 +1,44 @@
 <?php
 /**
- * @licence http://opensource.org/licenses/gpl-2.0.php GPL
+ * TODO short desc
+ *
+ * TODO long desc
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
+ * 
+ * @author      Olivier Bilodeau <obilodeau@inverse.ca>
+ * @copyright   2008-2010 Inverse inc.
+ * @license     http://opensource.org/licenses/gpl-2.0.php      GPL
  */
 
-
 require_once($_SERVER['DOCUMENT_ROOT'] . "/check_login.php");
 
+# these value to string are duplicated from lib/pf/config.pm
+# changes here should be reflected there
+$connection_type = array(
+  'Wireless-802.11-EAP'   => 'Wireless 802.1X (WPA Enterprise)',
+  'Wireless-802.11-NoEAP' => 'Wireless MAC Authentication',
+  'Ethernet-EAP'          => 'Wired 802.1x',
+  'Ethernet-NoEAP'        => 'Wired MAC Authentication',
+  'SNMP-Traps'            => 'Wired SNMP',
+  ''                      => 'Unknown'
+);
+
 if($sajax){
-	require($_SERVER['DOCUMENT_ROOT'] . "/common/sajax/Sajax.php");
+  require($_SERVER['DOCUMENT_ROOT'] . "/common/sajax/Sajax.php");
 }
 
   class table{
@@ -19,6 +50,7 @@ if($sajax){
     var $violationable;
     var $scannable;
     var $create_cmd;
+    var $count_cmd;
     var $linkable;
     var $hidden_links;
     var $is_hideable;
@@ -87,6 +119,10 @@ if($sajax){
       $this->default_filter=$filter;
    }
 
+   function set_count_cmd($count_cmd){
+      $this->count_cmd=$count_cmd;
+   }
+
    /**
     * Returns the number of displayed column in the table. 
     */
@@ -96,14 +132,24 @@ if($sajax){
    }
 
    function refresh(){
-     #$this=new table($this->create_cmd);
 
      $new_this = new table($this->create_cmd);
+     # if counting rows was done using a special command, we need to carry it over
+     if(isset($this->count_cmd) && $this->count_cmd != '') {
+       $new_this->set_count_cmd($this->count_cmd);
+     }
+
+     # copy over all members of new table to current table
      foreach (get_object_vars($new_this) as $key => $value)
        $this->$key = $value;  
 
+     # re-count number of rows if required
+     if(isset($this->count_cmd) && $this->count_cmd != '') {
+       $this->count_result();
+     }
+
      $this->set_editable(true);
-    }
+   }
 
     function get_key(){
       global $current_top;
@@ -111,11 +157,11 @@ if($sajax){
     
       if($current_top == 'status' && $current_sub == 'reports'){
         global $_GET;
-	$sub = "$current_top-$current_sub-$_GET[type]";   
+        $sub = "$current_top-$current_sub-$_GET[type]";   
       } 
 
       if(!$current_sub){
-	$menu = "$current_top-view";
+        $menu = "$current_top-view";
       }
       else{
         $menu = "$current_top-$current_sub";
@@ -144,10 +190,10 @@ if($sajax){
       for($i=1; $i<=count($content); $i++){
         if(isset($content[$i]) && $content[$i]!=""){
           $data=explode("|", $content[$i]);
-	  for($a=0; $a<count($this->headers); $a++)
-	    $row[$this->headers[$a]]=$data[$a];         
+          for($a=0; $a<count($this->headers); $a++)
+            $row[$this->headers[$a]]=$data[$a];         
           $this->rows[]=$row;
-  	}
+        }
       }   
     } // End constructor    
 
@@ -165,7 +211,7 @@ if($sajax){
         }
       }
       if(count($filtered_array)==0){
-	$this->is_empty=true;
+        $this->is_empty=true;
       }
     return $filtered_array;
     }
@@ -177,21 +223,21 @@ if($sajax){
       global $_GET;
       global $no_filter;
       global $extra_goodness;
-      $sort = 		$_GET['sort'];
-      $direction =	$_GET['direction'];
-      $per_page = 	$_GET['per_page'];
-      $filter = 	$_REQUEST['filter'];
-      $action = 	$_GET['action'];
-      $item = 		$_GET['item'];
-      $commit = 	$_POST['commit'];
-      $abs_url =	$_REQUEST['abs_url']; 
-      $time_filter = 	$_REQUEST['time_filter'];
-      $starttime = 	$_REQUEST['starttime'];
-      $stoptime = 	$_REQUEST['stoptime'];
+      $sort = $_GET['sort'];
+      $direction = $_GET['direction'];
+      $per_page = $_GET['per_page'];
+      $filter = $_REQUEST['filter'];
+      $action = $_GET['action'];
+      $item = $_GET['item'];
+      $commit = $_POST['commit'];
+      $abs_url =$_REQUEST['abs_url']; 
+      $time_filter = $_REQUEST['time_filter'];
+      $starttime = $_REQUEST['starttime'];
+      $stoptime = $_REQUEST['stoptime'];
 
       if(isset($this->hidden_links)){
         if(array_key_exists($sort, $this->hidden_links)){
-  	  $this->is_hidden=false;
+          $this->is_hidden=false;
         }
       }
 
@@ -214,21 +260,20 @@ if($sajax){
       print "<td colspan=\"".$this->get_displayable_column_count()."\" id=\"search\">\n";
       if($this->is_hideable){
         if($this->is_hidden){
-	           print "<span id='show_icon' style='display:visible;'><a href='javascript:hideCells(\"\");'><img src='../images/show.gif' alt='Show Info'><br><font size=1>Show Info</font></a></span>";
-	           print "<span id='hide_icon' style='display:none;'><a href='javascript:hideCells(\"none\");'><img src='../images/hide.gif' alt='Hide Info'><br><font size=1>Hide Info</font></a></span>";
+          print "<span id='show_icon' style='display:visible;'><a href='javascript:hideCells(\"\");'><img src='../images/show.gif' alt='Show Info'><br><font size=1>Show Info</font></a></span>";
+          print "<span id='hide_icon' style='display:none;'><a href='javascript:hideCells(\"none\");'><img src='../images/hide.gif' alt='Hide Info'><br><font size=1>Hide Info</font></a></span>";
+        } else {
+          print "<span id='show_icon' style='display:none;'><a href='javascript:hideCells(\"\");'><img src='../images/show.gif' alt='Show Info'><br><font size=1>Show Info</font></a></span>";
+          print "<span id='hide_icon' style='display:visible;'><a href='javascript:hideCells(\"none\");'><img src='../images/hide.gif' alt='Hide Info'><br><font size=1>Hide Info</font></a></span>";
         }
-	else{
-	           print "<span id='show_icon' style='display:none;'><a href='javascript:hideCells(\"\");'><img src='../images/show.gif' alt='Show Info'><br><font size=1>Show Info</font></a></span>";
-	           print "<span id='hide_icon' style='display:visible;'><a href='javascript:hideCells(\"none\");'><img src='../images/hide.gif' alt='Hide Info'><br><font size=1>Hide Info</font></a></span>";
-        }   
       }
 
       ## FILTER ANNEX ##
       if(!$with_add && !$no_filter && !($current_top=="scan" && $current_sub=="scan")){
-	if($current_top == 'status' && $current_sub == 'reports'){
-		global $type;
-		$t = "<input type='hidden' name='type' value='".trim($type)."'>";
-	}
+        if($current_top == 'status' && $current_sub == 'reports'){
+          global $type;
+          $t = "<input type='hidden' name='type' value='".trim($type)."'>";
+        }
         if (isset($this->default_filter) || (isset($filter) && $filter != '')) {
           if (!isset($filter) || $filter == '') {
             $last_filter = $this->default_filter;
@@ -257,22 +302,22 @@ if($sajax){
             if(!$stoptime)
               $stoptime='-Stop Date-';
             print "</form>\n";
-	    print "  <FORM name='timeform' action='/$current_top/$current_sub.php?filter=$filter'>";
+            print "  <FORM name='timeform' action='/$current_top/$current_sub.php?filter=$filter'>";
             print "<table>\n";
             print "<tr>\n";
-	    print "  <td></td>\n";
-	    print "  <td><input name='starttime' id='starttime' value='$starttime'></td>\n";
-	    show_calendar('starttime');
-   	    print "  <td></td>\n";  
-	    print "</tr>\n";
+            print "  <td></td>\n";
+            print "  <td><input name='starttime' id='starttime' value='$starttime'></td>\n";
+            show_calendar('starttime');
+            print "  <td></td>\n";  
+            print "</tr>\n";
             print "<tr>\n";
-	    print "  <td></td>\n";
-	    print "  <td><input name='stoptime' id='stoptime' value='$stoptime'></td>\n";
-	    show_calendar('stoptime');
-	    print "  <td></td>\n";	
-	    print "</tr>\n";
+            print "  <td></td>\n";
+            print "  <td><input name='stoptime' id='stoptime' value='$stoptime'></td>\n";
+            show_calendar('stoptime');
+            print "  <td></td>\n";
+            print "</tr>\n";
             print "<tr height='30'>";
-	    print "  <td valign='bottom'></td>";
+            print "  <td valign='bottom'></td>";
             print "  <td align='right'>";
             print "  <input type='submit' value='Submit'>";
             print "  </td>";
@@ -300,7 +345,7 @@ if($sajax){
       }
 
       if(!$current_sub){
-	$sub = "$current_top-view";
+        $sub = "$current_top-view";
       }
       else{
         $sub = "$current_top-$current_sub";
@@ -317,7 +362,7 @@ if($sajax){
       foreach($this->headers as $header){ 
         $pretty_header="";
 
-	if($header_meta){
+        if($header_meta){
           foreach($header_meta as $meta){
             if (($meta[0] == $header) || ($meta[0] == ($header . "*"))) {
               $pretty_header=$meta[1];
@@ -326,8 +371,8 @@ if($sajax){
         }
 
         if(!$pretty_header){
-	      $pretty_header=ucfirst($header);
-	}
+          $pretty_header=ucfirst($header);
+        }
 
         if (isset($this->default_sort_direction) || (isset($direction) && $direction != '')) { 
           if (!isset($sort) || $sort == '') {
@@ -349,9 +394,9 @@ if($sajax){
        isset($this->hidden_links[$header]) && $this->is_hidden == true ? $hide_tag = "id='id".++$q."' style='display:none;'" : $hide_tag = "";
 
        if($sort==$header)
-	  print "    <td class='header' $hide_tag><div class='header'><a class='active' href='$current_top/$current_sub.php?filter=" . urlencode($filter) . "&amp;sort=$header&amp;direction=$on_direction&amp;per_page=$per_page&$xtra_args'>$pretty_header</a></div></td>\n";
-	else
-	  print "    <td class='header' $hide_tag><div class='header'><a href='$current_top/$current_sub.php?filter=" . urlencode($filter) . "&amp;sort=$header&amp;direction=$off_direction&amp;per_page=$per_page&$xtra_args'>$pretty_header</a></div></td>\n";
+         print "    <td class='header' $hide_tag><div class='header'><a class='active' href='$current_top/$current_sub.php?filter=" . urlencode($filter) . "&amp;sort=$header&amp;direction=$on_direction&amp;per_page=$per_page&$xtra_args'>$pretty_header</a></div></td>\n";
+       else
+         print "    <td class='header' $hide_tag><div class='header'><a href='$current_top/$current_sub.php?filter=" . urlencode($filter) . "&amp;sort=$header&amp;direction=$off_direction&amp;per_page=$per_page&$xtra_args'>$pretty_header</a></div></td>\n";
       }
 
       print "  </tr>\n";
@@ -374,10 +419,10 @@ if($sajax){
  
   ## SET PAGE DEFAULTS ##
   if(!$this->per_page)
- 	  $this->per_page=25;
+    $this->per_page=25;
 
   if(!$this->page_num)
-	  $this->page_num=1;
+    $this->page_num=1;
 
   if(!$this->sql_sort_and_limit) {
       $start=($this->page_num - 1)*$this->per_page;
@@ -388,81 +433,81 @@ if($sajax){
 
   print "<tbody>\n";
   for($i=$start; $i<=$stop; $i++){
-	  if($i>=count($this->rows))
-		  break;
+    if($i>=count($this->rows))
+      break;
 
-  ## ROW HIGHLIGHTING ##	
+  ## ROW HIGHLIGHTING ##
   print "<tr class=\"data\">\n";
 
   ## EDITING A ROW ##
   if($action=="edit" && !$commit && $item==$i){
-	  print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page&amp;action=$action&amp;item=$item' method='post'>\n";
-	  $a=-1;
+    print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page&amp;action=$action&amp;item=$item' method='post'>\n";
+    $a=-1;
 
     foreach($this->rows[$i] as $cell){
-	     $key=$this->headers[++$a];
-	
-	     ## FOR AUTOSIZING ##
-	     $default_min=5;
-	     $default_max=15;
-	     $size_array=array();
-	     foreach($this->rows as $row)
-	       $size_array[]=strlen($row[$key]);	     
-	     $size=max(max($size_array), $default_min);
-	     $size=min($size, $default_max);
+      $key=$this->headers[++$a];
 
-             if(in_array($key, $this->headers))
-                print "<td><input size='$size' type='text' value='$cell' name='val$a'></td>\n";
-	     else
-               print "<td>$cell</td>"; 
-          }
+       ## FOR AUTOSIZING ##
+       $default_min=5;
+       $default_max=15;
+       $size_array=array();
+       foreach($this->rows as $row)
+         $size_array[]=strlen($row[$key]);
+       $size=max(max($size_array), $default_min);
+       $size=min($size, $default_max);
 
-           if($this->rows[$item+1])
-	     $value=implode("\t", $this->rows[$item+1]);
+       if(in_array($key, $this->headers))
+         print "<td><input size='$size' type='text' value='$cell' name='val$a'></td>\n";
+       else
+         print "<td>$cell</td>"; 
+    }
 
-           print "<input type='hidden' name='original' value='$value'>";
-           print "<input type='hidden' name='commit' value='true'>";
-           print "<td width='50'><div id='submit'><input type='submit' value='Submit'></div></td>\n";
-           print "</form>";
-  	 }
+    if($this->rows[$item+1])
+      $value=implode("\t", $this->rows[$item+1]);
 
-	 else{
-	   $a=-1;
-	   $key_item='';
+    print "<input type='hidden' name='original' value='$value'>";
+    print "<input type='hidden' name='commit' value='true'>";
+    print "<td width='50'><div id='submit'><input type='submit' value='Submit'></div></td>\n";
+    print "</form>";
+  }
+
+  else{
+    $a=-1;
+    $key_item='';
            
-           foreach($this->rows[$i] as $cell){
-             $key=$this->headers[++$a];
-	
-	     if($key == $this->key){
-	       $key_item=$cell;
-             }
-           }
+    foreach($this->rows[$i] as $cell){
+      $key=$this->headers[++$a];
+
+      if($key == $this->key){
+        $key_item=$cell;
+      }
+    }
 
            if(isset($this->editable)){
              print "  <td class=\"action\">\n";
              if (($current_top == 'configuration') && ($current_sub=='interfaces')) {
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['interface'] . "',500,500)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['interface'] . "',500,500)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
-	       print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['interface'] . "',500,500)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['interface'] . "',500,500)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
+               print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
                print "  <input type='hidden' name='action' value='delete'>\n";
                print "  <input type='hidden' name='commit' value='true'>\n";
                print "  <input type='hidden' name='original' value='".implode("\t", $this->rows[$i])."'>\n";
                print "  <input class=\"button\" type='image' src='/images/famfamfam_silk_icons/page_delete.png' align=bottom title='Delete this record' onClick=\"return confirm('Are you sure you want to delete the interface " . $this->rows[$i]['interface'] . " ?');\">\n";
                print "  </form>";
              } elseif (($current_top == 'configuration') && ($current_sub=='networks')) {
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['network'] . "',500,500)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['network'] . "',500,500)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
-	       print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['network'] . "',500,500)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['network'] . "',500,500)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
+               print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
                print "  <input type='hidden' name='action' value='delete'>\n";
                print "  <input type='hidden' name='commit' value='true'>\n";
                print "  <input type='hidden' name='original' value='".implode("\t", $this->rows[$i])."'>\n";
                print "  <input class=\"button\" type='image' src='/images/famfamfam_silk_icons/page_delete.png' align=bottom title='Delete this record' onClick=\"return confirm('Are you sure you want to delete the network " . $this->rows[$i]['network'] . " ?');\">\n";
                print "  </form>";
              } elseif (($current_top == 'configuration') && ($current_sub=='switches')) {
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['ip'] . "',500,500)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['ip'] . "',500,500)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['ip'] . "',500,500)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['ip'] . "',500,500)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
                if (($this->rows[$i]['ip'] != '127.0.0.1') && ($this->rows[$i]['ip'] != 'default')) {
-	         print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
+                 print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
                  print "  <input type='hidden' name='action' value='delete'>\n";
                  print "  <input type='hidden' name='commit' value='true'>\n";
                  print "  <input type='hidden' name='original' value='".implode("\t", $this->rows[$i])."'>\n";
@@ -494,10 +539,10 @@ if($sajax){
                  print "  </form>";
                }
              } elseif (($current_top == 'configuration') && ($current_sub=='violation')) {
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['vid'] . "',500,400)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
-	       print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['vid'] . "',500,400)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_edit.php?item=" . $this->rows[$i]['vid'] . "',500,400)\" title='Edit this record'><img src='/images/famfamfam_silk_icons/page_edit.png' alt=\"[ Edit ]\"></a>\n";
+               print "  <a href=\"javascript:popUp('/$current_top/" . $current_sub . "_add.php?item=" . $this->rows[$i]['vid'] . "',500,400)\" title='Clone this record'><img src='/images/famfamfam_silk_icons/page_add.png' alt=\"[ Add ]\"></a>\n";
                if (($this->rows[$i]['vid'] != '1100001') && ($this->rows[$i]['vid'] != 1100004) && ($this->rows[$i]['vid'] != 1100005) && ($this->rows[$i]['vid'] != 1100009) && ($this->rows[$i]['vid'] != 1100010) && ($this->rows[$i]['vid'] != 1200001) && ($this->rows[$i]['vid'] != 1200003) && ($this->rows[$i]['vid'] != 'defaults')) {
-	         print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
+                 print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page' method='post'>";
                  print "  <input type='hidden' name='action' value='delete'>\n";
                  print "  <input type='hidden' name='commit' value='true'>\n";
                  print "  <input type='hidden' name='original' value='".implode("\t", $this->rows[$i])."'>\n";
@@ -505,18 +550,18 @@ if($sajax){
                  print "  </form>";
                }
              } else {
-	       print "  <a href=\"javascript:popUp('/$current_top/edit.php?item=$key_item',500,400)\" title='Edit this record'><img src='/images/edit.png' alt=\"[ Edit ]\"></a>\n";
+               print "  <a href=\"javascript:popUp('/$current_top/edit.php?item=$key_item',500,400)\" title='Edit this record'><img src='/images/edit.png' alt=\"[ Edit ]\"></a>\n";
                if($this->violationable){
                  print "  <a href='violation/add.php?MAC=".$this->rows[$i]['mac']."'><img src='/images/trap.png' border='0' title='Add Violation' alt='[ Add Violation ]'></a>\n";
                }
-	       print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page&amp;action=$action&amp;item=$item' method='post'>";
+               print "<form action='/$current_top/$current_sub.php?filter=$filter&amp;sort=$sort&amp;direction=$direction&amp;page_num=$this->page_num&amp;per_page=$this->per_page&amp;action=$action&amp;item=$item' method='post'>";
                print "  <input type='hidden' name='action' value='delete'>\n";
                print "  <input type='hidden' name='commit' value='true'>\n";
                print "  <input type='hidden' name='original' value='".implode("\t", $this->rows[$i])."'>\n";
                print "  <input class=\"button\" type='image' src='/images/delete.png' align=bottom title='Delete this record' onClick=\"return confirm('Are you sure you want to delete ".$this->rows[$i][$this->key]."?');\">\n";
                print "  </form>";
              }
-   	     print "</td>\n";
+             print "</td>\n";
            }
        
            # FIXME: this is broken, nessus/scanner.php doesn't exist
@@ -525,34 +570,39 @@ if($sajax){
              $host=$this->rows[$i]['mac'];
              print "  <A HREF=\"javascript:popUp('nessus/scanner.php?host=$host')\">";
              print "  <input class=\"button\" type='image' align='center' src='/images/delete.png' onClick=\"return confirm('Scan this host?');\"></a>\n";
-     	     print "</td>\n";
+             print "</td>\n";
            }
 
-	   $a=-1;
+           $a=-1;
            foreach($this->rows[$i] as $cell){
              $key=$this->headers[++$a];
-	
-	     if($key == $this->key){
-	       $key_item=$cell;
+
+             if($key == $this->key){
+               $key_item=$cell;
              }
              isset($this->hidden_links[$key]) && $this->is_hidden == true ? $hide_tag = "id='id".++$q."' style='display:none;'" : $hide_tag = "";
 
              if(isset($this->linkable[$key])){
                strstr($this->linkable[$key], '?') ? $break = '&' : $break = '?';
 
-	       if($key == 'dhcp_fingerprint' && $_SESSION['fingerprints']["$cell"]){
+               if($key == 'dhcp_fingerprint' && $_SESSION['fingerprints']["$cell"]){
                  print "    <td $hide_tag><a href='".$this->linkable[$key].$break."view_item=$cell'>".$_SESSION['fingerprints']["$cell"]."</a></td>\n";
-	       }
+               }
                else if($key == 'vid' && $_SESSION['violation_classes']["$cell"]){
                  print "    <td $hide_tag><a href='".$this->linkable[$key].$break."view_item=$cell'>".$_SESSION['violation_classes']["$cell"]." </a></td>\n";
                } 
                else if (($key == 'url') && (array_key_exists('vid', $this->rows[$i]))) {
                  print "    <td $hide_tag><a href='".$this->linkable[$key].$break."vid=" . $this->rows[$i]['vid'] . "'>" . ((strlen($cell) > 30) ? (substr($cell, 0, 30) . ' ...') : $cell) . "</a></td>\n";
+
+               # HACK to support linking categories with space in their names
+               } else if ($current_top == 'node' && $current_sub == 'categories') {
+                 print "    <td $hide_tag><a href='".$this->linkable[$key].$break."view_item=\"$cell\"'>" . ((strlen($cell) > 30) ? (substr($cell, 0, 30) . ' ...') : $cell) . "</a></td>\n";
+
                }
-	       else{
+               else{
                  print "    <td $hide_tag><a href='".$this->linkable[$key].$break."view_item=$cell'>" . ((strlen($cell) > 30) ? (substr($cell, 0, 30) . ' ...') : $cell) . "</a></td>\n";
-	       }
-	     }
+               }
+             }
              else{  
                print "    <td $hide_tag>" . ((strlen($cell) > 30) ? (substr($cell, 0, 30) . ' ...') : $cell) . "</td>\n";
              }
@@ -594,7 +644,7 @@ if($sajax){
         $num_pages=ceil($this->result_count/$this->per_page);
       $next_page=$this->page_num+1;
       $last_page=$this->page_num-1;
-	
+
       if($num_pages>1) { // don't print the pager if there is only one page
       
         for($i=1; $i<=$num_pages; $i++){
@@ -626,12 +676,12 @@ if($sajax){
       }
       $per_pages=array('25', '25', '50', '100', '500', '1000');
       
-      if($this->per_page!=1001){   		# Because of report/history bug
+      if($this->per_page!=1001){   # Because of report/history bug
         for($a=1; $a<=count($per_pages); $a++){
           if($this->result_count>$per_pages[$a-1]){
-    	    if($this->per_page==$per_pages[$a])
+            if($this->per_page==$per_pages[$a])
               print "<a class='active' href='$current_top/$current_sub.php?sort=$sort&amp;direction=$direction&amp;per_page=$per_pages[$a]&amp;filter=" . urlencode($filter) . "&$xtra_args'>$per_pages[$a] </a>";
-	    else   
+            else   
               print "<a href='$current_top/$current_sub.php?sort=$sort&amp;direction=$direction&amp;per_page=$per_pages[$a]&amp;filter=" . urlencode($filter) . "&$xtra_args'>$per_pages[$a] </a>";
           }
         }
@@ -646,6 +696,17 @@ if($sajax){
     }  // End is_empty
 
 
+    /*
+     * Asks pfcmd to give us a count based on table's count_cmd
+     * I'm sorry it's a bit ugly but it's the best I could do in the context
+     */
+    function count_result() {
+      $result_count = PFCMD($this->count_cmd);
+      if ($result_count[1] >= 0) {
+        $this->set_result_count($result_count[1]);
+      }
+    }
+
   }  // End Class table
 
 
@@ -678,7 +739,7 @@ function PrintSubNav($menu){
         print "            </li>\n";
       }  
       else if($current_sub==$sub_nav[0]) {
-        print "        	<li class='active'><a href='$current_top/$sub_nav[0].php' class='current'>$sub_nav[1]</a></li>\n";
+        print "         <li class='active'><a href='$current_top/$sub_nav[0].php' class='current'>$sub_nav[1]</a></li>\n";
 
       } else {
         print "          <li><a href='$current_top/$sub_nav[0].php'>$sub_nav[1]</a></li>\n";
@@ -729,10 +790,18 @@ function PrintSubNav($menu){
       $headings[]=$heading[0];
       if (($current_top == 'node') && ($current_sub == 'add') && ($heading[0] == 'Status') ) {
         if (preg_match("/input type='text' name='(val\d+)'/", $heading[1], $regmatches)) {
-          $values[] = "<select name='" . $regmatches[1] . "'><option value='grace' >Grace</option><option value='reg'>Registered</option><option selected value='unreg'>Unregistered</option></select>";
+          $value_string = "<select name='" . $regmatches[1] . "'>";
+          # TODO: if printSelect would return a string instead of print directly this would be less ugly
+          $node_statuses = get_node_status_for_dropdown();
+          foreach ($node_statuses as $status_id => $status_name) {
+              $value_string .= "<option value='$status_id'>$status_name</option>";
+          }
+          $value_string .= "</select>";
+          $values[] = $value_string;
         } else {
           $values[]=$heading[1];
         }
+
       } elseif (($current_top == 'node') && ($current_sub == 'add') && ($heading[0] == 'Category') ) {
 
         if (preg_match("/input type='text' name='(val\d+)'/", $heading[1], $regmatches)) {
@@ -783,12 +852,12 @@ function PrintSubNav($menu){
       if($_REQUEST['action'] == 'add'){
         $add_info = PFCMD("$current_top view $_REQUEST[val0]");
         if($add_info[1]){ 
-  	  print "<tr><td colspan=2><b>Added Record</b></td></tr>";
-   	  $parts = explode('|', $add_info[1]);
+          print "<tr><td colspan=2><b>Added Record</b></td></tr>";
+          $parts = explode('|', $add_info[1]);
           //for($i=0; $i<count($parts); $i++){
           for($i=0; $i<1; $i++){
-	    print "<tr><td>$headings[$i]</td><td>$parts[$i]</td></tr>";
-	  }        
+            print "<tr><td>$headings[$i]</td><td>$parts[$i]</td></tr>";
+          }        
         }
         else{
           print "<tr><td><b><font color=red>Unable to add record $_REQUEST[val0]</b></font></td></tr>";
@@ -843,7 +912,7 @@ function PrintSubNav($menu){
     #exec("$PFCMD 2>&1", $output, $total);
 
     if(stristr($output[0], 'Usage: pfcmd')){
-      return false;	
+      return false;
     }
 
     # HACK: when the output of pfcmd has a "line 999" in it, we assume it's an error and we display it
@@ -855,8 +924,8 @@ function PrintSubNav($menu){
 
     if($errors){
       print "<div id='error' style='text-align:left;padding:10px;background:#FF7575;'>
-	<b>Error: Problems executing 'PFCMD $command'</b><br><pre>".
-	implode('<br>', $errors)."</pre></div>";
+        <b>Error: Problems executing 'PFCMD $command'</b><br><pre>".
+        implode('<br>', $errors)."</pre></div>";
       return false;
     }
 
@@ -874,7 +943,7 @@ function PrintSubNav($menu){
     foreach($meta_array as $data){
 
       if(preg_match("/^\-/", $data[0]))
-	continue;
+        continue;
   
       $i++;
       
@@ -890,8 +959,8 @@ function PrintSubNav($menu){
          $options=array_slice($options, 1, count($options)-2);
    
          $menu="<select name='val$i'>\n";
-	 foreach($options as $option)
- 	   $menu.="  <option value='$option'>$option\n";	
+         foreach($options as $option)
+           $menu.="  <option value='$option'>$option\n";
          $menu.="</select>"; 
       }
 
@@ -921,7 +990,7 @@ function PrintSubNav($menu){
 
   function jpgraph_check(){
     $jpgraph_dir = jpgraph_dir();
-	
+
     $extensions = get_loaded_extensions();
     if(!in_array('gd', $extensions)){ 
       print "<div id='error'>Error: PHP does not have GD installed.<br>JPGraph uses the graphing library GD to produce it's magnificent graphs, so you must install PHP with GD support.  For RedHat, use 'up2date php-gd' or 'yum php-gd'.</div>";
@@ -945,7 +1014,7 @@ function PrintSubNav($menu){
       $meta_array=meta("$current_top-$current_sub");
       if($meta_array){
         foreach($meta_array as $link){
-	  $link[0] == $current ? $links[]="<a href='$current_top/$current_sub.php?menu=true&type=$link[0]'><u>$link[1]</u></a>" : $links[]="<a href='$current_top/$current_sub.php?menu=true&type=$link[0]'>$link[1]</a>"; 
+          $link[0] == $current ? $links[]="<a href='$current_top/$current_sub.php?menu=true&type=$link[0]'><u>$link[1]</u></a>" : $links[]="<a href='$current_top/$current_sub.php?menu=true&type=$link[0]'>$link[1]</a>"; 
         }
        $o[] =  implode(" | ", $links);
       }
@@ -1024,8 +1093,8 @@ function PrintSubNav($menu){
         print "Could not open file: $global_conf<br>";
       }
       else{
- 	if(fwrite($DAT, serialize($defaults) === FALSE)){
-	  print "Couldn't write to file: $global_conf<br>";
+        if(fwrite($DAT, serialize($defaults) === FALSE)){
+          print "Couldn't write to file: $global_conf<br>";
         }
         fclose($DAT);
       }
@@ -1152,42 +1221,41 @@ function PrintSubNav($menu){
     print "<script type='text/javascript'>
              Calendar.setup(
              {
-               inputField  : '$id',       		    // ID of the input field
+               inputField  : '$id',               // ID of the input field
                ifFormat    : '%Y-%m-%d %H:%M:00', // the date format
-               button      : '$id',    			      // ID of the button
+               button      : '$id',               // ID of the button
                timeFormat  : \"24\",
                showsTime   : true
              }
-	   );
-	   </script>";
+             );
+           </script>";
   }
 
   function show_calendar_with_button($field,$button) {
     print "<script type='text/javascript'>
              Calendar.setup(
              {
-               inputField  : '$field',       		    // ID of the input field
+               inputField  : '$field',            // ID of the input field
                ifFormat    : '%Y-%m-%d %H:%M:00', // the date format
-               button      : '$button',    			      // ID of the button
+               button      : '$button',           // ID of the button
                timeFormat  : \"24\",
                showsTime   : true
              }
-	   );
-	   </script>";
+             );
+           </script>";
   }
 
   function show_calendar_with_button_without_time($field,$button) {
     print "<script type='text/javascript'>
              Calendar.setup(
              {
-               inputField  : '$field',       		    // ID of the input field
+               inputField  : '$field',   // ID of the input field
                ifFormat    : '%Y-%m-%d', // the date format
-               button      : '$button',    			      // ID of the button
+               button      : '$button',  // ID of the button
                timeFormat  : \"24\",
                showsTime   : false
              }
-	   );
-	   </script>";
+             );
+           </script>";
   }
 
-?>
