@@ -35,6 +35,63 @@ use pf::SNMP::constants;
 
 =cut
 
+=item supportsFloatingDevice
+
+Returns 1 if switch type supports floating network devices
+
+=cut
+sub supportsFloatingDevice {
+    my ( $this ) = @_;
+    my $logger = Log::Log4perl::get_logger( ref($this) );
+
+    $logger->error("Floating devices are not supported on switch type " . ref($this));
+    return $FALSE;
+}
+
+=item supportsWiredMacAuth 
+
+Returns 1 if switch type supports MAC Authentication (Wired Access Authorization through RADIUS)
+
+=cut
+sub supportsWiredMacAuth {
+    my ( $this ) = @_;
+    my $logger = Log::Log4perl::get_logger( ref($this) );
+
+    $logger->error(
+        "Wired MAC Authentication (Wired Access Authorization through RADIUS) "
+        . "is not supported on switch type " . ref($this) . ". Please let us know what hardware you are using."
+    );
+    return $FALSE;
+}
+
+=item supportsWiredDot1x - Returns 1 if switch type supports Wired 802.1X
+
+=cut
+sub supportsWiredDot1x {
+    my ( $this ) = @_;
+    my $logger = Log::Log4perl::get_logger( ref($this) );
+
+    $logger->error(
+        "Wired 802.1X is not supported on switch type " . ref($this) . ". "
+        . "Please let us know what hardware you are using."
+    );
+    return $FALSE;
+}
+
+=item supportsRadiusVoip
+
+=cut
+sub supportsRadiusVoip {
+    my ( $this ) = @_;
+    my $logger = Log::Log4perl::get_logger( ref($this) );
+
+    $logger->warn(
+        "RADIUS Authentication of IP Phones is not supported on switch type " . ref($this) . ". "
+        . "Please let us know what hardware you are using."
+    );
+    return $FALSE;
+}
+
 sub new {
     my ( $class, %argv ) = @_;
     my $this = bless {
@@ -49,8 +106,6 @@ sub new {
         '_dbUser'                   => undef,
         '_error'                    => undef,
         '_guestVlan'                => undef,
-        '_htaccessPwd'              => undef,
-        '_htaccessUser'             => undef,
         '_ip'                       => undef,
         '_isolationVlan'            => undef,
         '_macDetectionVlan'         => undef,
@@ -87,6 +142,10 @@ sub new {
         '_cliPwd'                   => undef,
         '_cliUser'                  => undef,
         '_cliTransport'             => undef,
+        '_wsPwd'                    => undef,
+        '_wsUser'                   => undef,
+        '_wsTransport'              => undef,
+        '_radiusSecret'             => undef,
         '_uplink'                   => undef,
         '_vlans'                    => undef,
         '_voiceVlan'                => undef,
@@ -122,10 +181,6 @@ sub new {
             $this->{_dbUser} = $argv{$_};
         } elsif (/^-?guestVlan$/i) {
             $this->{_guestVlan} = $argv{$_};
-        } elsif (/^-?htaccessPwd$/i) {
-            $this->{_htaccessPwd} = $argv{$_};
-        } elsif (/^-?htaccessUser$/i) {
-            $this->{_htaccessUser} = $argv{$_};
         } elsif (/^-?ip$/i) {
             $this->{_ip} = $argv{$_};
         } elsif (/^-?isolationVlan$/i) {
@@ -180,6 +235,14 @@ sub new {
             $this->{_cliUser} = $argv{$_};
         } elsif (/^-?cliTransport$/i) {
             $this->{_cliTransport} = $argv{$_};
+        } elsif (/^-?wsPwd$/i) {
+            $this->{_wsPwd} = $argv{$_};
+        } elsif (/^-?wsUser$/i) {
+            $this->{_wsUser} = $argv{$_};
+        } elsif (/^-?wsTransport$/i) {
+            $this->{_wsTransport} = lc($argv{$_});
+        } elsif (/^-?radiusSecret$/i) {
+            $this->{_radiusSecret} = lc($argv{$_});
         } elsif (/^-?uplink$/i) {
             $this->{_uplink} = $argv{$_};
         } elsif (/^-?SNMPEngineID$/i) {
@@ -386,7 +449,7 @@ sub disconnectWrite {
 =item connectMySQL - create MySQL database connection
 
 =cut
-
+# FIXME a connect but no disconnect? is this here useful at all?
 sub connectMySQL {
     my $this   = shift;
     my $logger = Log::Log4perl::get_logger( ref($this) );
@@ -423,9 +486,7 @@ sub connectMySQL {
 =cut
 
 sub setVlan {
-    my ( $this, $ifIndex, $newVlan, $switch_locker_ref, $presentPCMac,
-        $closeAllOpenLocationlogEntries )
-        = @_;
+    my ($this, $ifIndex, $newVlan, $switch_locker_ref, $presentPCMac) = @_;
     my $logger = Log::Log4perl::get_logger( ref($this) );
 
     if ( !$this->isProductionMode() ) {
@@ -446,11 +507,8 @@ sub setVlan {
         }
     }
 
-    #synchronize locationlog if necessary
-    locationlog_synchronize( $this->{_ip}, $ifIndex, $vlan, $presentPCMac );
-
     #handle some exceptions
-    if ( grep( { $_ == $newVlan } @{ $this->{_vlans} } ) == 0 )
+    if (!$this->isManagedVlan($vlan))
     {    #unmanaged VLAN ?
         $logger->warn(
             "new VLAN $newVlan is not a managed VLAN -> replacing VLAN $newVlan with MAC detection VLAN "
@@ -458,6 +516,9 @@ sub setVlan {
         $newVlan = $this->{_macDetectionVlan};
     }
 
+    #closes old locationlog entries and create a new one if required
+    locationlog_synchronize($this->{_ip}, $ifIndex, $newVlan, $presentPCMac, NO_VOIP, WIRED_SNMP_TRAPS);
+
     if ( !$this->isDefinedVlan($newVlan) ) {    #new VLAN is not defined
         if ( $newVlan == $this->{_macDetectionVlan} ) {
             $logger->warn( "MAC detection VLAN "
@@ -489,32 +550,15 @@ sub setVlan {
     }
 
     if ( $vlan == $newVlan ) {
-        $logger->info( "Should set "
-                . $this->{_ip}
-                . " ifIndex $ifIndex to VLAN $newVlan but it is already in this VLAN -> Do nothing"
+        $logger->info(
+            "Should set " . $this->{_ip}
+            . " ifIndex $ifIndex to VLAN $newVlan but it is already in this VLAN -> Do nothing"
         );
-
-   #        if ($closeAllOpenLocationlogEntries) {
-   #            locationlog_update_end($this->{_ip}, $ifIndex, $presentPCMac);
-   #        }
         return 1;
     }
 
-    #update locationlog
-    $logger->debug(
-        "updating locationlog for " . $this->{_ip} . " ifIndex $ifIndex" );
-    if ($closeAllOpenLocationlogEntries) {
-        locationlog_update_end( $this->{_ip}, $ifIndex, $presentPCMac );
-    } else {
-        locationlog_update_end_switchport_no_VoIP( $this->{_ip}, $ifIndex );
-    }
-    locationlog_insert_start( $this->{_ip}, $ifIndex, $newVlan,
-        $presentPCMac );
-
     #and finally set the VLAN
-    $logger->info( "setting VLAN at "
-            . $this->{_ip}
-            . " ifIndex $ifIndex from $vlan to $newVlan" );
+    $logger->info("setting VLAN at " . $this->{_ip} . " ifIndex $ifIndex from $vlan to $newVlan");
     return $this->_setVlan( $ifIndex, $newVlan, $vlan, $switch_locker_ref );
 }
 
@@ -561,12 +605,34 @@ sub _setVlanByOnlyModifyingPvid {
     return ( defined($result) );
 }
 
+=item getVlanByName - get the VLAN number of a given name in switches.conf
+ 
+Input: vlan name (as in switches.conf)
+
+=cut                    
+sub getVlanByName {
+    my ($this, $vlanName) = @_;
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    if (!exists($this->{"_".$vlanName})) {
+        # VLAN name doesn't exist
+        $logger->warn("VLAN $vlanName is not a valid VLAN identifier (see switches.conf)");
+        return;
+    }
+    
+    if ($this->{"_".$vlanName} !~ /^\d+$/) {
+        # is not resolved to a valid VLAN number
+        $logger->warn("VLAN $vlanName is not properly configured in switches.conf, not a vlan number");
+        return;
+    }   
+    return $this->{"_".$vlanName};
+}
+
 =item setVlanByName - set the ifIndex VLAN to the VLAN identified by given name in switches.conf
 
 Input: ifIndex, vlan name (as in switches.conf), switch lock
 
 =cut
-# TODO: get rid of the _ character in front of the vlan variables (refactoring)
 sub setVlanByName {
     my ($this, $ifIndex, $vlanName, $switch_locker_ref) = @_;
     my $logger = Log::Log4perl::get_logger(ref($this));
@@ -760,7 +826,7 @@ sub getManagedIfIndexes {
     foreach my $ifIndex ( keys %{$ifTypeHashRef} ) {
 
         # skip non ethernetCsmacd port type
-        if ( $ifTypeHashRef->{$ifIndex} == 6 ) {
+        if ( $ifTypeHashRef->{$ifIndex} == $SNMP::ETHERNET_CSMACD ) {
 
             # skip UpLinks
             if ( grep( { $_ == $ifIndex } @UpLinks ) == 0 ) {
@@ -768,7 +834,7 @@ sub getManagedIfIndexes {
 
                 # skip ports with ifOperStatus not present
                 if (   ( defined $ifOperStatus )
-                    && ( $ifOperStatusHashRef->{$ifIndex} != 6 ) )
+                    && ( $ifOperStatusHashRef->{$ifIndex} != $SNMP::NOT_PRESENT ) )
                 {
                     push @tmp_managedIfIndexes, $ifIndex;
                 } else {
@@ -810,6 +876,20 @@ sub getManagedIfIndexes {
     return @managedIfIndexes;
 }
 
+=item isManagedVlan - is the VLAN in the list of VLANs managed by the switch?
+
+=cut
+sub isManagedVlan {
+    my ($this, $vlan) = @_;
+
+    # can I find $vlan in _vlans ?
+    if (grep({$_ == $vlan} @{$this->{_vlans}}) == 0) {
+        #unmanaged VLAN
+        return 0;
+    }
+    return 1;
+}
+
 =item getMode - get the mode
 
 =cut
@@ -1081,11 +1161,11 @@ sub setAdminStatus {
     }
     $logger->trace(
         "SNMP set_request for ifAdminStatus: $OID_ifAdminStatus.$ifIndex = "
-            . ( $enabled ? 1 : 2 ) );
+            . ( $enabled ? $SNMP::UP : $SNMP::DOWN ) );
     my $result = $this->{_sessionWrite}->set_request(
         -varbindlist => [
             "$OID_ifAdminStatus.$ifIndex", Net::SNMP::INTEGER,
-            ( $enabled ? 1 : 2 ),
+            ( $enabled ? $SNMP::UP : $SNMP::DOWN ),
         ]
     );
     return ( defined($result) );
@@ -1130,14 +1210,6 @@ sub setPortSecurityViolationActionByIfIndex {
     return ( 0 == 1 );
 }
 
-sub supportsFloatingDevice {
-    my ( $this ) = @_;
-    my $logger = Log::Log4perl::get_logger( ref($this) );
-
-    $logger->error("Floating devices are not supported on switch type " . ref($this));
-    return 0;
-}   
-
 sub enablePortSecurityByIfIndex {
     my ( $this, $ifIndex ) = @_;
     my $logger = Log::Log4perl::get_logger( ref($this) );
@@ -1207,29 +1279,32 @@ sub getPhonesDPAtIfIndex {
     return @phones;
 }
 
+=item hasPhoneAtIfIndex
+
+Is there at least one IP Phone on the given ifIndex.
+
+=cut
 sub hasPhoneAtIfIndex {
     my ( $this, $ifIndex ) = @_;
     my $logger = Log::Log4perl::get_logger( ref($this) );
+
     if ( !$this->isVoIPEnabled() ) {
         $logger->debug( "VoIP not enabled on switch " . $this->{_ip} );
         return 0;
     }
-    my @macArray
-        = $this->_getMacAtIfIndex( $ifIndex, $this->getVoiceVlan($ifIndex) );
+
+    my @macArray = $this->_getMacAtIfIndex( $ifIndex, $this->getVoiceVlan($ifIndex) );
     foreach my $mac (@macArray) {
-        if ( !$this->isFakeMac($mac) ) {
-            $logger->trace("determining DHCP fingerprint info for $mac");
-            my $node_info = node_view_with_fingerprint($mac);
-            if ( defined($node_info)
-                && ( $node_info->{dhcp_fingerprint} =~ /VoIP Phone/ ) )
-            {
-                return 1;
-            }
+
+        if ($this->isPhoneAtIfIndex($mac, $ifIndex)) {
+            return 1;
         }
     }
-    $logger->trace( "determining through discovery protocols if "
-            . $this->{_ip}
-            . " ifIndex $ifIndex has VoIP phone connected" );
+
+    $logger->info(
+        "determining through discovery protocols if "
+        . $this->{_ip} . " ifIndex $ifIndex has VoIP phone connected"
+    );
     return ( scalar( $this->getPhonesDPAtIfIndex($ifIndex) ) > 0 );
 }
 
@@ -1251,25 +1326,32 @@ sub isPhoneAtIfIndex {
     $logger->trace("determining DHCP fingerprint info for $mac");
     my $node_info = node_view_with_fingerprint($mac);
 
-    #do we have node information
-    if ( defined($node_info) ) {
-        if ( $node_info->{dhcp_fingerprint} =~ /VoIP Phone/ ) {
-            $logger->debug("DHCP fingerprint for $mac indicates VoIP phone");
-            return 1;
-        }
+    if (defined($node_info->{'voip'}) && $node_info->{'voip'} eq VOIP) {
+        $logger->debug("This is a VoIP phone according to node.voip");
+        return 1;
+    }
 
-        #unknown DHCP fingerprint or no DHCP fingerprint
-        if ( $node_info->{dhcp_fingerprint} ne ' ' ) {
-            $logger->debug( "DHCP fingerprint for $mac indicates "
-                    . $node_info->{dhcp_fingerprint}
-                    . ". This is not a VoIP phone" );
-            return 0;
-        }
+    if (defined($node_info->{dhcp_fingerprint}) && $node_info->{dhcp_fingerprint} =~ /VoIP Phone/) {
+        $logger->debug("DHCP fingerprint for $mac indicates VoIP phone");
+        # returning 1 here will have the consequence of the node 
+        # updating node.voip to yes in locationlog_synchronize
+        return 1;
+    }
+
+    #unknown DHCP fingerprint or no DHCP fingerprint
+    if (defined($node_info->{dhcp_fingerprint}) && $node_info->{dhcp_fingerprint} ne ' ') {
+        $logger->debug(
+            "DHCP fingerprint for $mac indicates " .$node_info->{dhcp_fingerprint}. ". This is not a VoIP phone"
+        );
+    }
+
+    if (defined($ifIndex)) {
+        $logger->debug("determining if $mac is VoIP phone through discovery protocols");
+        my @phones = $this->getPhonesDPAtIfIndex($ifIndex);
+        return ( grep( { lc($_) eq lc($mac) } @phones ) != 0 );
+    } else {
+        return 0;
     }
-    $logger->trace(
-        "determining if $mac is VoIP phone through discovery protocols");
-    my @phones = $this->getPhonesDPAtIfIndex($ifIndex);
-    return ( grep( { lc($_) eq lc($mac) } @phones ) != 0 );
 }
 
 sub getMinOSVersion {
@@ -1376,6 +1458,7 @@ sub getRegExpFromList {
 The input must be the untranslated raw result of an snmp get_table
 
 =cut
+# TODO move out to a util package
 sub getBitAtPosition {
    my ($this, $bitStream, $position) = @_;
    return substr(unpack('B*', $bitStream), $position, 1);
@@ -1384,7 +1467,7 @@ sub getBitAtPosition {
 =item modifyBitmask - replaces the specified bit in a packed bitmask and returns the modified bitmask, re-packed
 
 =cut
-
+# TODO move out to a util package
 sub modifyBitmask {
     my ( $this, $bitMask, $offset, $replacement ) = @_;
     my $bitMaskString = unpack( 'B*', $bitMask );
@@ -1397,7 +1480,7 @@ sub modifyBitmask {
 The output is a packed binary representation useful to snmp::set_request
 
 =cut
-
+# TODO move out to a util package
 sub createPortListWithOneItem {
     my ($this, $position) = @_;
     
@@ -1411,7 +1494,7 @@ sub createPortListWithOneItem {
 Works on byte blocks since perl's bitewise not operates at the arithmetic level and some hardware have so many ports that I could overflow integers.
 
 =cut
-
+# TODO move out to a util package
 sub reverseBitmask {
     my ($this, $bitMask) = @_;
 
@@ -1778,19 +1861,19 @@ sub getHubs {
 sub getIfIndexForThisMac {
     my ( $this, $mac ) = @_;
     my $logger = Log::Log4perl::get_logger( ref($this) );
-    my @macParts = split( ':', $mac );
+    my $oid_mac = mac2oid($mac);
+
+    if (!defined($oid_mac)) {
+        $logger->warn("invalid MAC, not running request");
+        return -1;
+    }
     if ( !$this->connectRead() ) {
         return -1;
     }
+
+    my $oid_dot1qTpFdbPort = '1.3.6.1.2.1.17.7.1.2.2.1.2'; #Q-BRIDGE-MIB
     foreach my $vlan ( @{ $this->{_vlans} } ) {
-        my $oid
-            = "1.3.6.1.2.1.17.7.1.2.2.1.2.$vlan."
-            . hex( $macParts[0] ) . "."
-            . hex( $macParts[1] ) . "."
-            . hex( $macParts[2] ) . "."
-            . hex( $macParts[3] ) . "."
-            . hex( $macParts[4] ) . "."
-            . hex( $macParts[5] );
+        my $oid = "$oid_dot1qTpFdbPort.$vlan.$oid_mac";
         $logger->trace("SNMP get_request for $oid");
         my $result
             = $this->{_sessionRead}->get_request( -varbindlist => [$oid] );
@@ -1802,11 +1885,13 @@ sub getIfIndexForThisMac {
     return -1;
 }
 
+# TODO: unclear method contract
 sub getVmVlanType {
     my ( $this, $ifIndex ) = @_;
     return 1;
 }
 
+# TODO: unclear method contract
 sub setVmVlanType {
     my ( $this, $ifIndex, $type ) = @_;
     return 1;
@@ -1963,8 +2048,9 @@ sub isNewerVersionThan {
     return 0;
 }
 
+# TODO move out to a util package
 sub generateFakeMac {
-    my ( $this, $vlan, $ifIndex ) = @_;
+    my ($this, $is_voice_vlan, $ifIndex) = @_;
     my $logger = Log::Log4perl::get_logger(ref($this));
 
     # generating a fixed 6 digit string with ifIndex (zero filled)
@@ -1980,14 +2066,16 @@ sub generateFakeMac {
     }
 
     # VoIP will be different than non-VoIP
-    return "02:00:" . ( ( $vlan eq 'VoIP' ) ? "01" : "00" ) . ":" . $mac_suffix;
+    return "02:00:" . ( ($is_voice_vlan) ? "01" : "00" ) . ":" . $mac_suffix;
 }
 
+# TODO move out to a util package
 sub isFakeMac {
     my ( $this, $mac ) = @_;
     return ( $mac =~ /^02:00:00/ );
 }
 
+# TODO move out to a util package
 sub isFakeVoIPMac {
     my ( $this, $mac ) = @_;
     return ( $mac =~ /^02:00:01/ );
@@ -2016,6 +2104,7 @@ sub getUpLinks {
     return @upLinks;
 }
 
+# TODO: what the hell is this supposed to do?
 sub getVlanFdbId {
     my ( $this, $vlan ) = @_;
     my $OID_dot1qVlanFdbId = '1.3.6.1.2.1.17.7.1.4.2.1.3.0';    #Q-BRIDGE-MIB
@@ -2059,6 +2148,180 @@ sub setIfLinkUpDownTrapEnable {
     return ( defined($result) );
 }
 
+=item deauthenticateMac - performs wireless deauthentication
+
+mac - mac address to deauthenticate
+
+is_dot1x - set to 1 if special dot1x de-authentication is required
+
+=cut
+sub deauthenticateMac {
+    my ($this, $mac, $is_dot1x) = @_;
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    $logger->warn("Unimplemented! First, make sure your configuration is ok. "
+        . "If it is then we don't support your hardware. Open a bug report with your hardware type.");
+    return;
+}
+
+=item dot1xPortReauthenticate - forces 802.1x re-authentication of a given ifIndex
+
+ifIndex - ifIndex to force re-authentication on
+
+=cut
+sub dot1xPortReauthenticate {
+    my ($this, $ifIndex) = @_;
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    $logger->info("Trying generic MIB to force 802.1x port re-authentication. Your mileage may vary. "
+        . "If it doesn't work open a bug report with your hardware type.");
+ 
+    my $oid_dot1xPaePortReauthenticate = "1.0.8802.1.1.1.1.1.2.1.5"; # from IEEE8021-PAE-MIB
+
+    if (!$this->connectWrite()) {
+        return 0;
+    }
+
+    $logger->trace("SNMP set_request force dot1xPaePortReauthenticate on ifIndex: $ifIndex");
+    my $result = $this->{_sessionWrite}->set_request(-varbindlist => [
+        "$oid_dot1xPaePortReauthenticate.$ifIndex", Net::SNMP::INTEGER, 1
+    ]);
+
+    if (!defined($result)) {
+        $logger->error("got an SNMP error trying to force 802.1x re-authentication: ".$this->{_sessionWrite}->error);
+    }
+
+    return (defined($result));
+}
+
+=item NasPortToIfIndex 
+
+Translate RADIUS NAS-Port into the physical port ifIndex
+
+=cut
+sub NasPortToIfIndex {
+    my ($this, $NAS_port) = @_;
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    $logger->warn("This switch model doesn't seem to implement 802.1X or a degraded variant "
+        . "like MAC Authentication. Please let us know what hardware you are using");
+    return $NAS_port;
+}
+
+=item handleReAssignVlanTrapForWiredMacAuth
+
+Called when a ReAssignVlan trap is received for a switch-port in Wired MAC Authentication.
+
+Default behavior is to bounce the port if there's no IPT device on it.
+If there is and we should have isolated the device send an email to the admins.
+
+=cut
+sub handleReAssignVlanTrapForWiredMacAuth {
+    my ($this, $ifIndex) = @_;
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    my $switch_ip = $this->{'_ip'};
+    my @locationlog = locationlog_view_open_switchport_no_VoIP( $switch_ip, $ifIndex );
+    if (!(@locationlog) || !defined($locationlog[0]->{'mac'}) || ($locationlog[0]->{'mac'} eq '' )) {
+        $logger->warn( "received reAssignVlan trap on $switch_ip ifIndex $ifIndex but can't determine non VoIP MAC");
+        return;
+    }
+
+    my $mac = $locationlog[0]->{'mac'};
+    my $hasPhone = $this->hasPhoneAtIfIndex($ifIndex);
+
+    # TODO extract that behavior in a method call in pf::vlan so it can be overridden easily
+    if ( !$hasPhone ) {
+        $logger->info( "no VoIP phone is currently connected at " . $switch_ip
+            . " ifIndex $ifIndex. Flipping port admin status"
+        );
+        $this->setAdminStatus( $ifIndex, 0 );
+        sleep(2);
+        $this->setAdminStatus( $ifIndex, 1 );
+
+    } else {
+
+        $logger->info(
+            "A VoIP phone is currently connected at $switch_ip ifIndex $ifIndex. Leaving everything as it is."
+        ); 
+        # TODO perform CoA (when implemented)
+
+        my @violations = violation_view_open_desc($mac);
+        if ( scalar(@violations) > 0 ) {
+            my %message;
+            $message{'subject'} = "VLAN isolation of $mac behind VoIP phone";
+            $message{'message'} = "The following computer has been isolated behind a VoIP phone\n";
+            $message{'message'} .= "MAC: $mac\n";
+
+            my $node_info = node_view($mac);
+            $message{'message'} .= "Owner: " . $node_info->{'pid'} . "\n";
+            $message{'message'} .= "Computer Name: " . $node_info->{'computername'} . "\n";
+            $message{'message'} .= "Notes: " . $node_info->{'notes'} . "\n";
+            $message{'message'} .= "Switch: " . $switch_ip . "\n";
+            $message{'message'} .= "Port (ifIndex): " . $ifIndex . "\n\n";
+            $message{'message'} .= "The violation details are\n";
+
+            foreach my $violation (@violations) {
+                $message{'message'} .= "Description: "
+                    . $violation->{'description'} . "\n";
+                $message{'message'} .= "Start: "
+                    . $violation->{'start_date'} . "\n";
+            }
+            $logger->info(
+                "sending email to admin regarding isolation of $mac behind VoIP phone"
+            );
+            # put the use statement here because we'll be able to get rid of it when refactoring this piece
+            use pf::util;
+            pfmailer(%message);
+        }
+    }
+}
+
+
+=item extractSsid
+
+Find RADIUS SSID parameter out of RADIUS REQUEST parameters
+
+SSID are not provided by a standardized parameter name so we encapsulate that complexity here.
+If your AP is not supported look in /usr/share/freeradius/dictionary* for vendor specific attributes (VSA).
+
+=cut
+sub extractSsid {
+    my ($this, $radius_request) = @_;
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    if (defined($radius_request->{'Cisco-AVPair'})) {
+
+        if ($radius_request->{'Cisco-AVPair'} =~ /^ssid=(.*)$/) { # ex: Cisco-AVPair = "ssid=Inverse-Secure"
+            return $1;
+        } else {
+            $logger->info("Unable to extract SSID of Cisco-AVPair: ".$radius_request->{'Cisco-AVPair'});
+        }
+    }
+
+    $logger->warn(
+        "Unable to extract SSID for module " . ref($this) . ". SSID-based VLAN assignments won't work. "
+        . "Please let us know so we can add support for it."
+    );
+    return;
+}
+
+=item getVoipVSA
+
+Get Voice over IP RADIUS Vendor Specific Attribute (VSA).
+
+=cut
+sub getVoipVsa {
+    my ($this) = @_;
+    my $logger = Log::Log4perl::get_logger(ref($this));
+
+    $logger->warn(
+        "No RADIUS Vendor Specific Attributes (VSA) for module " . ref($this) . ". "
+        . "Phone will not be allowed on the correct untagged VLAN."
+    );
+    return;
+}
+
 =back
 
 =head1 AUTHOR
