@@ -11,6 +11,19 @@ to access SNMP enabled Aironet access points.
 
 =cut
 
+=head1 BUGS AND LIMITATIONS
+
+The same VLAN cannot be shared between two SSIDs
+
+The Vendor Specific Attributes (VSA) needs to be enabled for SSID identification to work.
+  radius-server vsa send authentication
+
+Wireless deauthentication (deassociation) uses the CLI (telnet or ssh) which is expensive (doesn't scale very well).
+
+Using Pre-Shared Key and MAC filtering (RADIUS MAC Authentication) is not possible on these devices.
+
+=cut
+
 use strict;
 use warnings;
 use diagnostics;
