@@ -17,6 +17,7 @@ use strict;
 use warnings;
 use diagnostics;
 
+use pf::config;
 use pf::util;
 use pf::iplog;
 use pf::node;
@@ -32,31 +33,31 @@ sub lookup_node {
     if ( node_exist($mac) ) {
 
         my $node_info = node_view($mac);
-        $return .= "MAC Address: $mac\n";
+        $return .= "MAC Address    : $mac\n";
 
         # fetch IP and DHCP information
         my $node_iplog_info = iplog_view_open_mac($mac);
         if (defined($node_iplog_info->{'ip'})) {
 
-            $return .= "IP Address : ".$node_iplog_info->{'ip'}." (active)\n";
-            $return .= "IP Info    : IP active since " . $node_iplog_info->{'start_time'} .
+            $return .= "IP Address     : ".$node_iplog_info->{'ip'}." (active)\n";
+            $return .= "IP Info        : IP active since " . $node_iplog_info->{'start_time'} .
                        " and DHCP lease valid until ".$node_iplog_info->{'end_time'}."\n";
             
         } else {
             my @node_iplog_history_info = iplog_history_mac($mac);
             if (ref($node_iplog_history_info[0]) eq 'HASH' && defined($node_iplog_history_info[0]->{'ip'})) {
                 my $latest_iplog = $node_iplog_history_info[0];
-                $return .= "IP Address : ".$latest_iplog->{'ip'}." (inactive)\n";
-                $return .= "IP Info    : IP was last seen active between " . $latest_iplog->{'start_time'} .
+                $return .= "IP Address     : ".$latest_iplog->{'ip'}." (inactive)\n";
+                $return .= "IP Info        : IP was last seen active between " . $latest_iplog->{'start_time'} .
                            " and ". $latest_iplog->{'end_time'} . "\n";
             } else {
-                $return .= "IP Address : Unknown\n";
-                $return .= "IP Info    : No IP information available\n";
+                $return .= "IP Address     : Unknown\n";
+                $return .= "IP Info        : No IP information available\n";
             }
         }
 
         # DHCP history
-        $return .= "DHCP Info  : Last DHCP request at ".$node_info->{'last_dhcp'}."\n";
+        $return .= "DHCP Info      : Last DHCP request at ".$node_info->{'last_dhcp'}."\n";
 
         my $owner  = $node_info->{'pid'};
         my $category = $node_info->{'category'};
@@ -72,22 +73,25 @@ sub lookup_node {
             $status = "grace";
         }
         $owner = "unregistered" if ( $owner eq '1' );
-        $return .= "Owner      : $owner\n"  if ($owner);
-        $return .= "Category   : $category\n" if ($category);
-        $return .= "Status     : $status\n" if ($status);
-        $return .= "Name       : " . $node_info->{'computername'} . "\n"
+        $return .= "Owner          : $owner\n"  if ($owner);
+        $return .= "Category       : $category\n" if ($category);
+        $return .= "Status         : $status\n" if ($status);
+        $return .= "Name           : " . $node_info->{'computername'} . "\n"
             if ( $node_info->{'computername'} );
-        $return .= "Notes      : " . $node_info->{'notes'} . "\n"
+        $return .= "Notes          : " . $node_info->{'notes'} . "\n"
             if ( $node_info->{'notes'} );
 
         my $vendor = oui_to_vendor($mac);
         if ($vendor) {
-            $return .= "MAC Vendor : $vendor\n";
+            $return .= "MAC Vendor     : $vendor\n";
         }
 
+        my $voip = $node_info->{'voip'};
+        $return .= "VoIP           : $voip\n" if ($voip);
+
         # TODO: output useragent class like in dhcp fingerprint
 
-        $return .= "Browser    : " . $node_info->{'user_agent'} . "\n"
+        $return .= "Browser        : " . $node_info->{'user_agent'} . "\n"
             if ( $node_info->{'user_agent'} );
 
         if ( $node_info->{'dhcp_fingerprint'} ) {
@@ -96,7 +100,7 @@ sub lookup_node {
             if ( scalar(@fingerprint_info_array == 1) ) {
                 my $fingerprint_info = $fingerprint_info_array[0];
                 my $os = $fingerprint_info->{'os'};
-                $return .= "OS         : $os\n" if ( defined($os) );
+                $return .= "OS             : $os\n" if ( defined($os) );
             }
         }
 
@@ -108,22 +112,37 @@ sub lookup_node {
                 if (defined($last_locationlog_entry[0]->{'end_time'}) && $last_locationlog_entry[0]->{'end_time'} !~ /0000/) {
                     $is_entry_active = 0;
                 }
-                $return .= "Location: port ". $last_locationlog_entry[0]->{'port'}." "
+                $return .= "Location       : port ". $last_locationlog_entry[0]->{'port'}." "
                         .  " (vlan " . $last_locationlog_entry[0]->{'vlan'}.")"
                         .  " on switch ".$last_locationlog_entry[0]->{'switch'}
                         .  "\n";
-                if ($is_entry_active) {
-                    $return .= "Last activity: currently active\n";
+
+                if (exists($last_locationlog_entry[0]->{'connection_type'})) {
+                    $return .= "Connection type: ".$last_locationlog_entry[0]->{'connection_type'}."\n";
                 } else {
-                    $return .= "Last activity: ".$last_locationlog_entry[0]->{'end_time'}."\n";
+                    $return .= "Connection type: UNKNOWN\n";
+                }
+
+                if (defined($last_locationlog_entry[0]->{'dot1x_username'})) {
+                    $return .= "802.1X Username: ".$last_locationlog_entry[0]->{'dot1x_username'}."\n";
+                }
+
+                if (defined($last_locationlog_entry[0]->{'ssid'})) {
+                    $return .= "Wireless SSID  : ".$last_locationlog_entry[0]->{'ssid'}."\n";
+                }
+
+                if ($is_entry_active) {
+                    $return .= "Last activity  : currently active\n";
+                } else {
+                    $return .= "Last activity  : ".$last_locationlog_entry[0]->{'end_time'}."\n";
                 }
             } else {
                 $return .= "No connectivity information available (We probably only saw a DHCP request)\n";
             }
         } else {
-            my $port   = $node_info->{'port'};
-            my $switch = $node_info->{'switch'};
-            my $vlan   = $node_info->{'vlan'};
+            my $port   = $node_info->{'last_port'};
+            my $switch = $node_info->{'last_switch'};
+            my $vlan   = $node_info->{'last_vlan'};
             my $switch_ip;
             my $switch_mac;
             if ($switch) {
@@ -188,3 +207,7 @@ USA.
 =cut
 
 1;
+
+# vim: set shiftwidth=4:
+# vim: set expandtab:
+# vim: set backspace=indent,eol,start:
