@@ -38,6 +38,7 @@ floatingnetworkdeviceconfig  | query/modify floating network device configuratio
 fingerprint                  | view DHCP Fingerprints
 graph                        | trending graphs
 history                      | IP/MAC history
+import                       | bulk import of information into the database
 ifoctetshistorymac           | accounting history
 ifoctetshistoryswitch        | accounting history
 ifoctetshistoryuser          | accounting history
@@ -382,9 +383,9 @@ Usage: pfcmd trigger view <id> [scan|detect]
 view the Snort IDs, OS IDs,  and Nessus plugins associated with violations.  To edit, modify violations.conf and execute "pfcmd reload violations" 
 
 examples:
-	pfcmd trigger view all
-	pfcmd trigger view 12
-	pfcmd trigger view all scan
+  pfcmd trigger view all
+  pfcmd trigger view 12
+  pfcmd trigger view all scan
 EOT
     return 1;
 }
@@ -528,6 +529,23 @@ EOT
     return 1;
 }   
 
+sub help_import {
+    print STDERR << "EOT";
+Usage: pfcmd import <nodes> <filename>
+
+Bulk import into the database. File input must be a of CSV format.
+
+Node import format:
+<MAC>
+
+Node import automatically registers MACs with pid = 1
+
+example:
+  pfcmd import nodes /tmp/new-nodes.csv
+EOT
+    return 1;
+}
+
 =head1 AUTHOR
 
 David LaPorte <david@davidlaporte.org>
