@@ -22,6 +22,7 @@ use POSIX();
 use Net::SMTP;
 use Net::MAC::Vendor;
 use Log::Log4perl;
+# TODO these look like they are not required.. get rid of them?
 use threads;
 use threads::shared;
 
@@ -37,7 +38,9 @@ BEGIN {
         isdisabled getlocalmac ip2int int2ip get_all_internal_ips get_internal_nets get_routed_isolation_nets get_routed_registration_nets get_internal_ips
         get_internal_devs get_internal_devs_phy get_external_devs get_managed_devs get_internal_macs
         get_internal_info get_gateways get_dhcp_devs createpid readpid deletepid
-        pfmon_preload parse_template mysql_date oui_to_vendor mac2oid oid2mac);
+        pfmon_preload parse_template mysql_date oui_to_vendor mac2oid oid2mac str_to_connection_type 
+        connection_type_to_str
+    );
 }
 
 use pf::config;
@@ -92,21 +95,35 @@ sub valid_ip {
     }
 }
 
+=item clean_mac 
+
+Clean a MAC address accepting xx-xx-xx-xx-xx-xx, xx:xx:xx:xx:xx:xx and xxxx.xxxx.xxxx.
+
+Returns a string with MAC in format: xx:xx:xx:xx:xx:xx
+
+=cut
 sub clean_mac {
     my ($mac) = @_;
     return (0) if ( !$mac );
     $mac =~ s/\s//g;
     $mac = lc($mac);
-    $mac =~ s/\.//g if ( $mac =~ /^([0-9a-f]{4}(\.|$)){4}$/i );
+    $mac =~ s/\.//g if ( $mac =~ /^([0-9a-f]{4}(\.|$)){3}$/i );
     $mac =~ s/([a-f0-9]{2})(?!$)/$1:/g if ( $mac =~ /^[a-f0-9]{12}$/i );
     $mac = join q {:} => map { sprintf "%02x" => hex } split m {:|\-} => $mac;
     return ($mac);
 }
 
+=item valid_mac 
+
+Validates MAC addresses. Returns 1 or 0 (true or false)
+
+Accepting xx-xx-xx-xx-xx-xx, xx:xx:xx:xx:xx:xx and xxxx.xxxx.xxxx
+
+=cut
 sub valid_mac {
     my ($mac) = @_;
     my $logger = Log::Log4perl::get_logger('pf::util');
-    if (! ($mac =~ /^[0-9a-f:\.]+$/i)) {
+    if (! ($mac =~ /^[0-9a-f:\.-]+$/i)) {
         $logger->error("invalid MAC: $mac");
         return (0);
     }
@@ -727,6 +744,57 @@ sub preload_is_internal {
     return (%is_internal);
 }
 
+=item * connection_type_to_str
+
+In the database we store the connection type as a string but we use a constant binary value internally. 
+This converts from the constant binary value to the string.
+
+return connection_type string (as defined in pf::config) or an empty string if connection type not found
+
+=cut 
+sub connection_type_to_str {
+    my ($conn_type) = @_;
+    my $logger = Log::Log4perl::get_logger('pf::util');
+
+    # convert connection_type constant into a string for database
+    if (defined($conn_type) && $conn_type ne '' && defined($connection_type_to_str{$conn_type})) {
+
+        return $connection_type_to_str{$conn_type};
+    } else {
+        my ($package, undef, undef, $routine) = caller(1);
+        $logger->warn("unable to convert connection_type to string. called from $package $routine");
+        return '';
+    }
+}
+
+=item * str_to_connection_type
+
+In the database we store the connection type as a string but we use a constant binary value internally. 
+This parses the string from the database into the the constant binary value.
+
+return connection_type constant (as defined in pf::config) or undef if connection type not found
+
+=cut
+sub str_to_connection_type {
+    my ($conn_type_str) = @_;
+    my $logger = Log::Log4perl::get_logger('pf::util');
+
+    # convert database string into connection_type constant
+    if (defined($conn_type_str) && $conn_type_str ne '' && defined($connection_type{$conn_type_str})) {
+
+        return $connection_type{$conn_type_str};
+    } elsif (defined($conn_type_str) && $conn_type_str eq '') {
+
+        $logger->debug("got an empty connection_type, this happens if we discovered the node but it never connected");
+        return 'Unknown';
+
+    } else {
+        my ($package, undef, undef, $routine) = caller(1);
+        $logger->warn("unable to parse string into a connection_type constant. called from $package $routine");
+        return;
+    }
+}
+
 =back
 
 =head1 AUTHOR
@@ -743,7 +811,7 @@ Copyright (C) 2005 David LaPorte
 
 Copyright (C) 2005 Kevin Amorin
 
-Copyright (C) 2009 Inverse inc.
+Copyright (C) 2009,2010 Inverse inc.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
