@@ -20,6 +20,14 @@ Read the F<pf.conf> configuration file.
 use strict;
 use warnings;
 use Log::Log4perl;
+use Readonly;
+
+# Violation status constants
+#TODO port all hard-coded strings to these constants
+#but first I need to resolve the exporting problems.. 
+#ex: when trying to use these from node I get subroutines redefinitions 
+#    and if I use full package names, there is no safety that the constant was defined in the first place..
+Readonly::Scalar our $STATUS_OPEN => 'open';
 
 use constant VIOLATION => 'violation';
 
@@ -94,8 +102,12 @@ sub violation_db_prepare {
     $violation_statements->{'violation_view_sql'} = get_db_handle()->prepare(
         qq [ select violation.id,violation.mac,node.computername,violation.vid,violation.start_date,violation.release_date,violation.status,violation.ticket_ref,violation.notes from violation,node where violation.mac=node.mac and violation.id=? order by start_date desc ]);
 
-    $violation_statements->{'violation_view_all_sql'} = get_db_handle()->prepare(
-        qq [ select violation.id,violation.mac,node.computername,violation.vid,violation.start_date,violation.release_date,violation.status,violation.ticket_ref,violation.notes from violation,node where violation.mac=node.mac ]);
+    $violation_statements->{'violation_view_all_sql'} = get_db_handle()->prepare(qq[
+        SELECT violation.id,violation.mac,node.computername,violation.vid,violation.start_date,violation.release_date,violation.status,violation.ticket_ref,violation.notes
+        FROM violation,node 
+        WHERE violation.mac=node.mac
+        ORDER BY start_date DESC
+    ]);
 
     $violation_statements->{'violation_view_open_sql'} = get_db_handle()->prepare(
         qq [ select id,mac,vid,start_date,release_date,status,ticket_ref,notes from violation where mac=? and status="open" order by start_date desc ]);
@@ -109,8 +121,12 @@ sub violation_db_prepare {
     $violation_statements->{'violation_view_open_all_sql'} = get_db_handle()->prepare(
         qq [ select id,mac,vid,start_date,release_date,status,ticket_ref,notes from violation where status="open" ]);
 
-    $violation_statements->{'violation_view_top_sql'} = get_db_handle()->prepare(
-        qq [ select id,mac,v.vid,start_date,release_date,status,ticket_ref,notes from violation v, class c where v.vid=c.vid and mac=? and status="open" order by priority desc limit 1]);
+    $violation_statements->{'violation_view_top_sql'} = get_db_handle()->prepare(qq[
+        SELECT id, mac, v.vid, start_date, release_date, status, ticket_ref, notes 
+        FROM violation v, class c 
+        WHERE v.vid=c.vid AND mac=? AND status="open" 
+        ORDER BY priority ASC LIMIT 1
+    ]);
 
     $violation_statements->{'violation_view_all_active_sql'} = get_db_handle()->prepare(
         qq [ select v.mac,v.vid,v.start_date,v.release_date,v.status,v.ticket_ref,v.notes,i.ip,i.start_time,i.end_time from violation v left join iplog i on v.mac=i.mac where v.status="open" and i.end_time=0 group by v.mac]);
@@ -302,7 +318,6 @@ sub violation_add {
     $data{notes}  = ""     if ( !defined $data{notes} );
     $data{ticket_ref} = "" if ( !defined $data{ticket_ref} );
 
-    # Is this MAC and ID aready in DB?  if so don't add another
     if ( violation_exist_open( $mac, $vid ) ) {
         $logger->info("violation $vid already exists for $mac, not adding again");
         return (1);
@@ -329,17 +344,15 @@ sub violation_add {
         }
     }
 
-    #  has this mac registered if not register for violation?
+    # if this node doesn't exist we'll run into problems so create it
     if ( !node_exist($mac) ) {
         node_add_simple($mac);
     } else {
 
-        # not a new violation check violation
+        # check if we are under the grace period of a previous violation
         my ($remaining_time) = violation_grace( $mac, $vid );
         if ( $remaining_time > 0 ) {
-            $logger->info(
-                "$remaining_time grace remaining on violation $vid for node $mac"
-            );
+            $logger->info("$remaining_time grace remaining on violation $vid for node $mac. Not adding violation.");
             return (1);
         } else {
             $logger->info("grace expired on violation $vid for node $mac");
@@ -370,24 +383,24 @@ sub violation_trigger {
     $type = lc($type);
 
     if (whitelisted_mac($mac)) {
-        $logger->info("violation not added, $mac is whitelisted! trigger id: $tid");
+        $logger->info("violation not added, $mac is whitelisted! trigger $type::$tid");
         return 0;
     } 
 
     if (!valid_mac($mac)) {
-        $logger->info("violation not added, MAC $mac is whitelisted! trigger id: $tid");
+        $logger->info("violation not added, MAC $mac is whitelisted! trigger $type::$tid");
         return 0;
     } 
 
     if (!trappable_mac($mac)) {
-        $logger->info("violation not added, MAC $mac is not trappable! trigger id: $tid");
+        $logger->info("violation not added, MAC $mac is not trappable! trigger $type::$tid");
         return 0;
     }
 
     # if we were given an IP as additionnal violation trigger info
     # test whether this ip is trappable or not
     if (defined($data{ip}) && !trappable_ip($data{ip})) {
-        $logger->info("violation not added, IP ".$data{ip}." is not trappable! trigger id: $tid, MAC: $mac");
+        $logger->info("violation not added, IP ".$data{ip}." is not trappable! trigger $type::$tid, MAC: $mac");
         return 0;
     }
 
@@ -414,22 +427,27 @@ sub violation_trigger {
         }
         my $vid = $row->{'vid'};
 
-        # TODO: performance improvement: validate for grace here also will avoid a spawned process only to check grace
-
         # if the node's category is whitelisted, skip
         if (_is_node_category_whitelisted($row, $mac)) {
             $logger->info("Not adding violation ${vid} node $mac is immune because of its category");
             next;
         }
 
-        # Is this MAC and ID aready in DB?  if so don't add another
         # we test here AND in violation_add because here we avoid a fork (and violation_add is called from elsewhere)
         if ( violation_exist_open( $mac, $vid ) ) {
-            $logger->info("violation $vid already exists for $mac, not adding again");
+            $logger->info("violation $vid (trigger $type::$tid) already exists for $mac, not adding again");
             next;
         }
 
-        $logger->info("calling $bin_dir/pfcmd violation add vid=$vid,mac=$mac");
+        # check if we are under the grace period of a previous violation
+        # we test here AND in violation_add because here we avoid a fork (and violation_add is called from elsewhere)
+        my ($remaining_time) = violation_grace( $mac, $vid );
+        if ($remaining_time > 0) {
+            $logger->info("$remaining_time grace remaining on violation $vid (trigger $type::$tid) for node $mac. Not adding violation.");
+            next;
+        }
+
+        $logger->info("calling '$bin_dir/pfcmd violation add vid=$vid,mac=$mac' (trigger $type::$tid)");
         # forking a pfcmd because it will call a vlan flip if needed
         `$bin_dir/pfcmd violation add vid=$vid,mac=$mac`;
         $addedViolation = 1;
@@ -569,3 +587,7 @@ USA.
 =cut
 
 1;
+
+# vim: set shiftwidth=4:
+# vim: set expandtab:
+# vim: set backspace=indent,eol,start:
