@@ -11,6 +11,9 @@ pf::web - module to generate the different web pages.
 pf::web contains the functions necessary to generate different web pages:
 based on pre-defined templates: login, registration, release, error, status.  
 
+It is possible to customize the behavior of this module by redefining its subs in pf::web::custom.
+See F<pf::web::custom> for details.
+
 =head1 CONFIGURATION AND ENVIRONMENT
 
 Read the following template files: F<release.html>, 
@@ -19,6 +22,7 @@ F<register.html>.
 
 =cut
 
+#TODO all template destination should be variables allowing redefinitions by pf::web::custom
 use strict;
 use warnings;
 use Date::Parse;
@@ -32,9 +36,8 @@ BEGIN {
     use Exporter ();
     our ( @ISA, @EXPORT );
     @ISA = qw(Exporter);
-    @EXPORT = qw(generate_release_page generate_login_page generate_enabler_page generate_redirect_page 
-                 generate_error_page generate_status_page generate_registration_page web_node_register 
-                 web_node_record_user_agent web_user_authenticate generate_scan_start_page generate_scan_status_page);
+    # No export to force users to use full package name and allowing pf::web::custom to redefine us
+    @EXPORT = qw();
 }
 
 use pf::config;
@@ -43,6 +46,14 @@ use pf::iplog qw(ip2mac);
 use pf::node qw(node_view node_modify);
 use pf::useragent;
 
+=head1 SUBROUTINES
+
+Warning: The list of subroutine is incomplete
+
+=over
+
+=cut
+
 sub web_get_locale {
     my ($cgi,$session) = @_;
     my $logger = Log::Log4perl::get_logger('pf::web');
@@ -285,7 +296,7 @@ sub generate_scan_status_page {
         refresh_timer    => $refresh_timer
     };
 
-    my $ip = $cgi->remote_addr;
+    my $ip = get_client_ip($cgi);
     push @{ $vars->{list_help_info} },
         { name => gettext('IP'), value => $ip };
     my $mac = ip2mac($ip);
@@ -331,7 +342,7 @@ sub generate_error_page {
         $vars->{txt_message} = gettext($error_msg);
     }
 
-    my $ip = $cgi->remote_addr;
+    my $ip = get_client_ip($cgi);
     push @{ $vars->{list_help_info} },
         { name => gettext('IP'), value => $ip };
     my $mac = ip2mac($ip);
@@ -373,7 +384,7 @@ sub generate_status_page {
     setlocale( LC_MESSAGES, web_get_locale($cgi, $session) );
     bindtextdomain( "packetfence", "$conf_dir/locale" );
     textdomain("packetfence");
-    my $ip   = $cgi->remote_addr;
+    my $ip   = get_client_ip($cgi);
     my $vars = {
         logo            => $Config{'general'}{'logo'},
         txt_page_title  => gettext('Status'),
@@ -433,10 +444,25 @@ sub generate_status_page {
     exit;
 }
 
+=item web_node_register
+
+This sub is meant to be redefined by pf::web::custom to fit your specific needs.
+See F<pf::web::custom> for examples.
+
+=cut
 sub web_node_register {
+    my ( $cgi, $session, $mac, $pid, %info ) = @_;
+    my $logger = Log::Log4perl::get_logger('pf::web');
+
+    # we are good, push the registration
+    return _sanitize_and_register($mac, $pid, %info);
+}
+
+sub _sanitize_and_register {
     my ( $mac, $pid, %info ) = @_;
     my $logger = Log::Log4perl::get_logger('pf::web');
     my $info;
+
     foreach my $key ( keys %info ) {
         $info{$key} =~ s/[^0-9a-zA-Z_\*\.\-\:_\;\@\ ]/ /g;
         $info .= $key . '="' . $info{$key} . '",';
@@ -454,8 +480,7 @@ sub web_node_record_user_agent {
     my $logger = Log::Log4perl::get_logger('pf::web');
     
     # Recording useragent
-    $logger->info("Updating node $mac user_agent");
-    $logger->debug("Updating node $mac user_agent with useragent: '$user_agent'");
+    $logger->info("Updating node $mac user_agent with useragent: '$user_agent'");
     # call node_modify directly instead of pfcmd node edit. it's more performant and it'll avoid trying to adjust vlan 
     node_modify($mac, ('user_agent' => $user_agent));
 
@@ -464,7 +489,10 @@ sub web_node_record_user_agent {
     if ( scalar(@user_agent_info) && ( ref( $user_agent_info[0] ) eq 'HASH' ) ) {
 
         # is there a violation on this user agent?
-        $logger->debug("sending USERAGENT::".$user_agent_info[0]->{'useragent_id'}." (".$user_agent_info[0]->{'useragent'}.") trigger");
+        $logger->debug(
+            "sending USERAGENT::".$user_agent_info[0]->{'useragent_id'}
+            ." (".$user_agent_info[0]->{'useragent'}.") trigger"
+        );
         require pf::violation;
         pf::violation::violation_trigger( $mac, $user_agent_info[0]->{'useragent_id'}, "USERAGENT" );
 
@@ -478,6 +506,7 @@ sub web_node_record_user_agent {
 
 sub web_user_authenticate {
 
+    # TODO extract these magic digits in constants
     # return (1,0) for successfull authentication
     # return (0,2) for inability to check credentials
     # return (0,1) for wrong login/password
@@ -497,7 +526,6 @@ sub web_user_authenticate {
         }
 
         #validate login and password
-        use lib $conf_dir;
         eval "use authentication::$auth";
         if ($@) {
             $logger->error("ERROR loading authentication::$auth $@");
@@ -524,7 +552,7 @@ sub generate_registration_page {
     textdomain("packetfence");
     my $cookie = $cgi->cookie( CGISESSID => $session->id );
     print $cgi->header( -cookie => $cookie );
-    my $ip   = $cgi->remote_addr;
+    my $ip   = get_client_ip($cgi);
     my $vars = {
         logo            => $Config{'general'}{'logo'},
         deadline        => $Config{'registration'}{'skip_deadline'},
@@ -621,6 +649,74 @@ sub generate_registration_page {
     exit;
 }
 
+=item generate_pending_page
+
+Shows a page to user saying registration is pending.
+
+=cut
+sub generate_pending_page {
+    my ( $cgi, $session, $destination_url, $mac ) = @_;
+    setlocale( LC_MESSAGES, web_get_locale($cgi, $session) );
+    bindtextdomain( "packetfence", "$conf_dir/locale" );
+    textdomain("packetfence");
+    my $ip = $cgi->remote_addr;
+    my $vars = {
+        logo            => $Config{'general'}{'logo'},
+        txt_page_title  => "Registration pending",
+        txt_page_header => "Registration pending",
+        txt_help        => gettext('help: provide info'),
+        list_help_info  => [
+            { name => gettext('IP'),  value => $ip },
+            { name => gettext('MAC'), value => $mac }
+        ],
+    };
+
+    my $cookie = $cgi->cookie( CGISESSID => $session->id );
+    print $cgi->header( -cookie => $cookie );
+
+    my $template = Template->new(
+        { INCLUDE_PATH => ["$install_dir/html/user/content/templates"], } );
+    $template->process("pending.html", $vars);
+    exit;
+}
+
+=item get_client_ip
+
+Returns IP address of the client reaching the captive portal. 
+Either directly connected or through a proxy.
+
+=cut
+sub get_client_ip {
+    my ($cgi) = @_;
+    my $logger = Log::Log4perl::get_logger('pf::web');
+
+    $logger->trace("request for client IP");
+
+    # we fetch CGI's remote address
+    # if user is behind a proxy it's not sufficient since we'll get the proxy's IP
+    my $directly_connected_ip = $cgi->remote_addr();
+
+    # handling most common case first
+    if ($directly_connected_ip ne LOOPBACK_IPV4) {
+        return $directly_connected_ip;
+    }
+
+    # proxied?
+    if (defined($ENV{'HTTP_X_FORWARDED_FOR'})) {
+        my $proxied_ip = $ENV{'HTTP_X_FORWARDED_FOR'};
+        $logger->debug(
+            "Remote Address is ".LOOPBACK_IPV4.". Client is proxied? "
+            . "Returning: $proxied_ip according to HTTP Headers"
+        );
+        return $proxied_ip;
+    }
+
+    $logger->debug("Remote Address is ".LOOPBACK_IPV4." but no further hints of client IP in HTTP Headers");
+    return $directly_connected_ip;
+}
+
+=back
+
 =head1 AUTHOR
 
 David LaPorte <david@davidlaporte.org>
@@ -637,7 +733,7 @@ Copyright (C) 2005 David LaPorte
 
 Copyright (C) 2005 Kevin Amorin
 
-Copyright (C) 2008-2009 Inverse inc.
+Copyright (C) 2008-2010 Inverse inc.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
